SystemOrganization addCategory: #'PetitParser-Core'!
SystemOrganization addCategory: #'PetitParser-Builder'!
SystemOrganization addCategory: #'PetitParser-Tests'!

!SequenceableCollection methodsFor: '*petitparser' stamp: 'lr 4/19/2008 19:58'!
asParser
	^ self 
		inject: PPSequenceParser new
		into: [ :result :each | result , each asParser ]! !

!Set methodsFor: '*petitparser' stamp: 'lr 4/19/2008 19:58'!
asParser
	^ self 
		inject: PPChoiceParser new
		into: [ :result :each | result | each asParser ]! !

!Symbol methodsFor: '*petitparser' stamp: 'lr 4/20/2008 14:01'!
asParser
	^ PPPredicateParser perform: self! !

ReadStream subclass: #PPStream
	instanceVariableNames: 'cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPStream methodsFor: 'accessing' stamp: 'lr 4/22/2008 23:05'!
for: aParser do: aBlock
	| memento |
	memento := (cache
		at: aParser
		ifAbsentPut: [ IdentityDictionary new ])
			at: position
			ifAbsentPut: [ PPMemento new ].
	memento result isNil 
		ifFalse: [ position := memento position ]
		ifTrue: [
			memento increment.
			memento result: (readLimit - position + 1 < memento count
				ifTrue: [ PPFailure reason: 'Overflow' position: position ]
				ifFalse: [ aBlock value ]).
			memento position: position ].
	^ memento result! !

!PPStream methodsFor: 'initialization' stamp: 'lr 4/24/2008 10:10'!
initialize
	cache := IdentityDictionary new! !

!PPStream methodsFor: 'private' stamp: 'lr 4/21/2008 16:35'!
on: aCollection
	self initialize.
	super on: aCollection! !

!PPStream methodsFor: 'private' stamp: 'lr 4/21/2008 16:35'!
on: aCollection from: firstIndex to: lastIndex
	self initialize.
	super on: aCollection from: firstIndex to: lastIndex! !

!PPStream methodsFor: 'accessing' stamp: 'lr 4/22/2008 22:47'!
peek
	^ self atEnd ifFalse: [ collection at: position + 1 ]! !

!PositionableStream methodsFor: '*petitparser' stamp: 'lr 4/19/2008 13:17'!
collection
	^ collection! !

!Object methodsFor: '*petitparser' stamp: 'lr 4/20/2008 16:06'!
asParser
	^ PPPredicateParser expect: self! !

!Object methodsFor: '*petitparser-testing' stamp: 'lr 4/18/2008 13:40'!
isFailure
	^ false! !

Object subclass: #PPFailure
	instanceVariableNames: 'reason position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPFailure class methodsFor: 'instance-creation' stamp: 'lr 4/18/2008 14:17'!
reason: aString
	^ self new reason: aString! !

!PPFailure class methodsFor: 'instance-creation' stamp: 'lr 4/19/2008 09:24'!
reason: aString position: anInteger
	^ self new reason: aString; position: anInteger! !

!PPFailure methodsFor: 'testing' stamp: 'lr 4/18/2008 13:41'!
isFailure
	^ true! !

!PPFailure methodsFor: 'accessing' stamp: 'lr 4/19/2008 09:24'!
position
	^ position! !

!PPFailure methodsFor: 'accessing' stamp: 'lr 4/19/2008 09:24'!
position: anInteger
	position := anInteger! !

!PPFailure methodsFor: 'printing' stamp: 'lr 4/18/2008 12:04'!
printOn: aStream
	aStream nextPutAll: reason! !

!PPFailure methodsFor: 'accessing' stamp: 'lr 4/19/2008 09:24'!
reason
	^ reason! !

!PPFailure methodsFor: 'accessing' stamp: 'lr 4/18/2008 14:18'!
reason: aString
	reason := aString! !

Object subclass: #PPMemento
	instanceVariableNames: 'result count position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPMemento class methodsFor: 'as yet unclassified' stamp: 'lr 4/22/2008 18:21'!
new
	^ self basicNew initialize! !

!PPMemento methodsFor: 'accessing' stamp: 'lr 4/22/2008 18:23'!
count
	^ count! !

!PPMemento methodsFor: 'accessing' stamp: 'lr 4/22/2008 18:25'!
count: anInteger
	count := anInteger! !

!PPMemento methodsFor: 'action' stamp: 'lr 4/22/2008 18:20'!
increment
	count := count + 1! !

!PPMemento methodsFor: 'initialization' stamp: 'lr 4/22/2008 18:21'!
initialize
	count := 0
	! !

!PPMemento methodsFor: 'accessing' stamp: 'lr 4/22/2008 18:23'!
position
	^ position! !

!PPMemento methodsFor: 'accessing' stamp: 'lr 4/26/2008 15:48'!
position: anInteger
	position := anInteger! !

!PPMemento methodsFor: 'accessing' stamp: 'lr 4/24/2008 10:15'!
result
	^ result! !

!PPMemento methodsFor: 'accessing' stamp: 'lr 4/22/2008 18:23'!
result: anObject
	result := anObject! !

Object subclass: #PPParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

PPParser subclass: #PPDelegateParser
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPDelegateParser commentStamp: 'lr 4/19/2008 12:57' prior: 0!
A parser that delegates to another parser.!

PPDelegateParser subclass: #PPActionParser
	instanceVariableNames: 'block'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPActionParser commentStamp: 'lr 4/19/2008 12:55' prior: 0!
A parser that performs an action on the delegate.!

!PPActionParser methodsFor: 'parsing' stamp: 'lr 4/21/2008 16:24'!
basicParse: aStream
	| element |
	element := super basicParse: aStream.
	^ element isFailure
		ifTrue: [ element ]
		ifFalse: [ block value: element ]! !

!PPActionParser methodsFor: 'accessing' stamp: 'lr 4/18/2008 14:02'!
block: aBlock
	block := aBlock! !

PPDelegateParser subclass: #PPCompositeParser
	instanceVariableNames: 'productions unresolved'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Builder'!

!PPCompositeParser class methodsFor: 'rules' stamp: 'lr 4/21/2008 07:36'!
transformParser
	<transform: 100>

	^ DSLConditionPattern new
		if: [ :visitor | visitor tree isMethod and: [ visitor tree selector isUnary and: [ visitor theClass ~= PPCompositeParser ] ] ]
		then: (Array
			with: (DSLSearchPattern new
				expression: '`#var';
				action: [ :visitor |
					(visitor node value isString or: [ visitor node value isCharacter ])
						ifTrue: [ visitor node replaceWith: ``(`,(visitor node value) asParser) ] ])
			with: (DSLSearchPattern new
				expression: '`var';
				action: [ :visitor |
					((visitor theClass bindingOf: visitor node name) isNil
						and: [ (visitor tree allDefinedVariables includes: visitor node name) not
						and: [ (visitor theClass allInstVarNames includes: visitor node name) not ] ])
						ifTrue: [ visitor node replaceWith: (``(self selector) selector: visitor node name asSymbol) ] ])
			with: [ :visitor |
				visitor tree body lastIsReturn ifTrue: [
					visitor tree body statements last
						replaceWith: visitor tree body statements last value ].
				visitor tree body addNodeFirst: ``(self
					productionAt: `,(visitor tree selector)
					definition: [ `,(visitor tree body) ]).
				visitor tree body statements allButFirst
					do: [ :each | visitor tree body removeNode: each ].
				visitor tree body addReturn ])! !

!PPCompositeParser methodsFor: 'initialization' stamp: 'lr 4/22/2008 22:48'!
initialize
	super initialize.
	productions := Dictionary new.
	unresolved := OrderedCollection new.
	parser := self startProduction.
	self resolve! !

!PPCompositeParser methodsFor: 'accessing' stamp: 'lr 4/26/2008 15:47'!
productionAt: aSymbol definition: aBlock
	productions 
		at: aSymbol
		ifPresent: [ :production | ^ production ].
	productions
		at: aSymbol
		put: (unresolved add: (PPUnresolvedParser on: aSymbol)).
	^ productions
		at: aSymbol
		put: aBlock value! !

!PPCompositeParser methodsFor: 'actions' stamp: 'lr 4/26/2008 15:47'!
resolve
	| resolved |
	unresolved := unresolved asArray.
	resolved := unresolved collect: [ :each | productions at: each symbol ].
	unresolved elementsForwardIdentityTo: resolved.
	unresolved := #()! !

!PPCompositeParser methodsFor: 'accessing' stamp: 'lr 4/21/2008 06:53'!
startProduction
	"Answer the production to start this parser with."
	
	self subclassResponsibility! !

PPCompositeParser subclass: #PPExpressionParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Builder'!

!PPExpressionParser methodsFor: 'productions' stamp: 'lr 4/25/2008 18:31'!
expr
		(expr , $+ token , term ==> [ :node | node first + node third ]) 
	|	(expr , $- token , term ==> [ :node | node first - node third ])
	|	(term)! !

!PPExpressionParser methodsFor: 'productions' stamp: 'lr 4/25/2008 18:33'!
fact
		($( token , expr , $) token ==> [ :node | node second ]) 
	| 	(num)! !

!PPExpressionParser methodsFor: 'productions' stamp: 'lr 4/22/2008 23:06'!
num
	($- optional , #digit plus , ($. , #digit plus) optional) token
		==> [ :node | Number readFrom: node copy ]! !

!PPExpressionParser methodsFor: 'accessing' stamp: 'lr 4/25/2008 15:41'!
startProduction
	expr end! !

!PPExpressionParser methodsFor: 'productions' stamp: 'lr 4/25/2008 18:42'!
term
		(term , $* token , fact ==> [ :node | node first * node third ]) 
	|	(term , $/ token , fact ==> [ :node | node first / node third ])
	|	(fact)! !

PPCompositeParser subclass: #PPLambdaParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Builder'!

!PPLambdaParser methodsFor: 'productions' stamp: 'lr 4/21/2008 07:14'!
abstraction
	$\ , variable , $. , expression! !

!PPLambdaParser methodsFor: 'productions' stamp: 'lr 4/21/2008 07:57'!
application
	expression , expression! !

!PPLambdaParser methodsFor: 'productions' stamp: 'lr 4/20/2008 11:46'!
expression
	variable | abstraction | application! !

!PPLambdaParser methodsFor: 'accessing' stamp: 'lr 4/25/2008 18:27'!
startProduction
	expression end! !

!PPLambdaParser methodsFor: 'productions' stamp: 'lr 4/21/2008 07:38'!
variable
	#letter plus token! !

PPCompositeParser subclass: #PPSchemeParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Builder'!

!PPSchemeParser methodsFor: 'productions-expressions' stamp: 'lr 4/23/2008 00:18'!
application
	open , selector , expression star , close ==> [ :node | 
		RBMessageNode 
			qqReceiver: node third first 
			selector: node second 
			arguments: node third allButFirst ]! !

!PPSchemeParser methodsFor: 'productions-literals' stamp: 'lr 4/23/2008 00:04'!
boolean
		('#t' token ==> [ :node | ``(true) ])
	|	('#f' token ==> [ :node | ``(false) ])! !

!PPSchemeParser methodsFor: 'tokens' stamp: 'lr 4/22/2008 23:43'!
close
	$) token! !

!PPSchemeParser methodsFor: 'productions' stamp: 'lr 4/23/2008 00:01'!
expression
		number
	|	string
	|	boolean
	|	identifier
	|	lambda
	|	application
		
	

! !

!PPSchemeParser methodsFor: 'productions-literals' stamp: 'lr 4/24/2008 10:31'!
identifier
	#letter plus token ==> [ :node | RBVariableNode named: node copy ]! !

!PPSchemeParser methodsFor: 'productions-expressions' stamp: 'lr 4/22/2008 23:58'!
if
		(open , 'if' token , expression , expression , close)
	|	(open , 'if' token , expression , expression , expression , close)! !

!PPSchemeParser methodsFor: 'productions-expressions' stamp: 'lr 4/24/2008 10:31'!
lambda
	(open , 'lambda' token , identifier , expression , close) ==> [ :node |
		``[ `,(node fourth) ] arguments: (Array with: ``(`,(node third))) ]! !

!PPSchemeParser methodsFor: 'productions-expressions' stamp: 'lr 4/22/2008 23:42'!
let
	! !

!PPSchemeParser methodsFor: 'productions-literals' stamp: 'lr 4/24/2008 10:30'!
number
	($- optional , #digit plus , ($. , #digit plus) optional) token
		==> [ :node | ``(`,(Number readFrom: node copy)) ]! !

!PPSchemeParser methodsFor: 'tokens' stamp: 'lr 4/22/2008 23:43'!
open
	$( token! !

!PPSchemeParser methodsFor: 'productions-literals' stamp: 'lr 4/24/2008 10:31'!
selector
	#space not plus token ==> [ :node | node copy asSymbol ]! !

!PPSchemeParser methodsFor: 'productions' stamp: 'lr 4/25/2008 18:27'!
startProduction
	expression end! !

!PPSchemeParser methodsFor: 'productions-literals' stamp: 'lr 4/24/2008 10:31'!
string
	($" , $" not star , $") token 
		==> [ :node | ``(`,(node copyFrom: 2 to: node size - 1)) ]! !

!PPDelegateParser class methodsFor: 'instance-creation' stamp: 'lr 4/20/2008 16:22'!
on: aParser
	^ self new setParser: aParser! !

!PPDelegateParser methodsFor: 'parsing' stamp: 'lr 4/21/2008 16:24'!
basicParse: aStream 
	^ parser parse: aStream! !

!PPDelegateParser methodsFor: 'initialization' stamp: 'lr 4/20/2008 16:23'!
setParser: aParser
	parser := aParser! !

PPDelegateParser subclass: #PPStarParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPStarParser commentStamp: 'lr 4/19/2008 12:56' prior: 0!
A parser that parses zero or more instances of my delegate.!

!PPStarParser methodsFor: 'parsing' stamp: 'lr 4/21/2008 16:28'!
basicParse: aStream
	| position elements element |
	elements := OrderedCollection new.
	[ position := aStream position.
	element := super basicParse: aStream.
	element isFailure ifTrue: [ 
		aStream position: position.
		^ elements asArray ].
	elements add: element ] repeat! !

PPDelegateParser subclass: #PPTokenParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPTokenParser commentStamp: 'lr 4/19/2008 12:54' prior: 0!
A parser that produces a token from the range my delegate parses.!

!PPTokenParser methodsFor: 'parsing' stamp: 'lr 4/24/2008 10:18'!
basicParse: aStream
	| start stop element |
	start := aStream position.
	element := super basicParse: aStream.
	element isFailure ifTrue: [ ^ element ].
	stop := aStream position.
	[ aStream atEnd not and: [ aStream peek isSeparator ] ]
		whileTrue: [ aStream next ].
	^ aStream collection sliceFrom: start + 1 to: stop! !

PPParser subclass: #PPEndOfInputParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPEndOfInputParser commentStamp: 'lr 4/18/2008 13:46' prior: 0!
A parser that succeeds only at the end of the input stream.!

!PPEndOfInputParser methodsFor: 'parsing' stamp: 'lr 4/19/2008 10:03'!
parse: aStream
	^ aStream atEnd 
		ifFalse: [ PPFailure reason: 'end of input expected' position: aStream position ]! !

PPParser subclass: #PPEpsilonParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPEpsilonParser commentStamp: 'lr 4/18/2008 15:35' prior: 0!
A parser that always succeeds.!

!PPEpsilonParser methodsFor: 'parsing' stamp: 'lr 4/18/2008 14:13'!
parse: aStream
	^ nil! !

PPParser subclass: #PPFailingParser
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPFailingParser class methodsFor: 'instance-creation' stamp: 'lr 4/19/2008 09:57'!
message: aString
	^ self new message: aString! !

!PPFailingParser methodsFor: 'parsing' stamp: 'lr 4/21/2008 16:29'!
basicParse: aStream
	^ PPFailure reason: message position: aStream position! !

!PPFailingParser methodsFor: 'accessing' stamp: 'lr 4/19/2008 09:56'!
message: aString
	message := aString! !

PPParser subclass: #PPListParser
	instanceVariableNames: 'parsers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

PPListParser subclass: #PPChoiceParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPChoiceParser commentStamp: 'lr 4/18/2008 15:35' prior: 0!
A parser that uses the first parser that succeeds.!

!PPChoiceParser methodsFor: 'parsing' stamp: 'lr 4/21/2008 16:25'!
basicParse: aStream
	| item |
	parsers do: [ :parser |
		item := parser parse: aStream.
		item isFailure
			ifFalse: [ ^ item ] ].
	^ item! !

!PPChoiceParser methodsFor: 'operations' stamp: 'lr 4/18/2008 10:53'!
| aRule 
	parsers add: aRule! !

!PPListParser methodsFor: 'initialization' stamp: 'lr 4/18/2008 10:53'!
initialize
	super initialize.
	parsers := OrderedCollection new! !

PPListParser subclass: #PPSequenceParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPSequenceParser commentStamp: 'lr 4/18/2008 15:34' prior: 0!
A parser that parses a sequence of parsers.!

!PPSequenceParser methodsFor: 'operations' stamp: 'lr 4/18/2008 10:53'!
, aRule 
	parsers add: aRule! !

!PPSequenceParser methodsFor: 'parsing' stamp: 'lr 4/21/2008 16:26'!
basicParse: aStream
	| index item items position |
	index := 1.
	position := aStream position.
	items := Array new: parsers size.
	parsers do: [ :parser |
		item := parser parse: aStream.
		item isFailure ifTrue: [
			aStream position: position.
			^ item ].
		items at: index put: item.
		index := index + 1 ].
	^ items! !

!PPParser class methodsFor: 'instance-creation' stamp: 'lr 4/18/2008 14:00'!
new
	^ self basicNew initialize! !

!PPParser methodsFor: 'operations' stamp: 'lr 4/19/2008 19:58'!
, aParser 
	"Answer a new parser that parses the receiver followed by aParser."

	^ PPSequenceParser new , self , aParser! !

!PPParser methodsFor: 'operations' stamp: 'lr 4/19/2008 10:05'!
==> aBlock
	"Assigns aBlock as a success action handler."
	
	^ (PPActionParser on: self) block: aBlock! !

!PPParser methodsFor: 'converting' stamp: 'lr 4/19/2008 13:08'!
asParser
	^ self! !

!PPParser methodsFor: 'parsing' stamp: 'lr 4/21/2008 16:23'!
basicParse: aStream
	self subclassResponsibility! !

!PPParser methodsFor: 'operations' stamp: 'lr 4/26/2008 15:50'!
end
	"Ensure the end of the input and return the result of the receiver."

	^ PPSequenceParser new , self , PPEndOfInputParser new ==> #first! !

!PPParser methodsFor: 'initialization' stamp: 'lr 4/24/2008 10:33'!
initialize! !

!PPParser methodsFor: 'testing' stamp: 'lr 4/20/2008 16:30'!
isUnresolved
	^ false! !

!PPParser methodsFor: 'operations' stamp: 'lr 4/21/2008 06:49'!
optional
	"Answer a new parser that parses the receiver, if possible."

	^ PPChoiceParser new | self | PPEpsilonParser new! !

!PPParser methodsFor: 'parsing' stamp: 'lr 4/21/2008 16:23'!
parse: aStream
	^ aStream for: self do: [ self basicParse: aStream ]! !

!PPParser methodsFor: 'operations' stamp: 'lr 4/25/2008 15:40'!
plus
	"Answer a new parser that parses the receiver one or more times."

	^ PPSequenceParser new , self , self star ==> [ :value | (Array with: value first) , value second ]! !

!PPParser methodsFor: 'operations' stamp: 'lr 4/19/2008 10:07'!
star
	"Answer a new parser that parses the receiver zero or more times."

	^ PPStarParser on: self! !

!PPParser methodsFor: 'operations' stamp: 'lr 4/19/2008 12:48'!
token
	"Answer a new parser that parses a token."
	
	^ PPTokenParser on: self! !

!PPParser methodsFor: 'operations' stamp: 'lr 4/19/2008 19:59'!
| aParser 
	"Answer a new parser that either parses the receiver or aParser."
	
	^ PPChoiceParser new | self | aParser! !

PPParser subclass: #PPPredicateParser
	instanceVariableNames: 'predicate predicateMessage negated negatedMessage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Core'!

!PPPredicateParser commentStamp: 'lr 4/18/2008 15:34' prior: 0!
A parser that accepts if a given predicate holds.!

!PPPredicateParser class methodsFor: 'factory-objects' stamp: 'lr 4/19/2008 11:21'!
any
	^ self
		on: [ :each | true ] message: 'something expected'
		negated: [ :each | false ] message: 'nothing expected'! !

!PPPredicateParser class methodsFor: 'factory-objects' stamp: 'lr 4/19/2008 11:25'!
between: min and: max
	^ self
		on: [ :each | each >= min and: [ each <= max ] ] message: min printString , '..' , max printString , ' expected'
		negated: [ :each | each < min or: [ each > max ] ] message: min printString , '..' , max printString , ' not expected'! !

!PPPredicateParser class methodsFor: 'factory-chars' stamp: 'lr 4/20/2008 15:54'!
char: aCharacter
	^ self expect: aCharacter! !

!PPPredicateParser class methodsFor: 'factory-chars' stamp: 'lr 4/19/2008 11:28'!
control
	^ self
		on: [ :each | each asInteger < 32 ] message: 'control character expected'
		negated: [ :each | each asInteger >= 32 ] message: 'no control character expected'! !

!PPPredicateParser class methodsFor: 'factory-chars' stamp: 'lr 4/19/2008 11:35'!
digit
	^ self 
		on: [ :each | each isDigit ] message: 'digit expected'
		negated: [ :each | each isDigit not ] message: 'no digit expected'! !

!PPPredicateParser class methodsFor: 'factory-objects' stamp: 'lr 4/19/2008 11:21'!
expect: anObject
	^ self 
		on: [ :each | each = anObject ] message: anObject printString , ' expected'
		negated: [ :each | each ~= anObject ] message: anObject printString , ' not expected'! !

!PPPredicateParser class methodsFor: 'factory-chars' stamp: 'lr 4/19/2008 11:35'!
letter
	^ self 
		on: [ :each | each isLetter ] message: 'letter expected'
		negated: [ :each | each isLetter not ] message: 'no letter expected'! !

!PPPredicateParser class methodsFor: 'factory-chars' stamp: 'lr 4/19/2008 11:28'!
lowercase
	^ self
		on: #isLowercase message: 'lowercase letter expected'
		negated: #isUppercase message: 'uppercase letter expected'! !

!PPPredicateParser class methodsFor: 'instance-creation' stamp: 'lr 4/19/2008 11:21'!
on: aBlock message: aString
	^ self on: aBlock message: aString negated: [ :each | (aBlock value: each) not ] message: 'no ' , aString! !

!PPPredicateParser class methodsFor: 'instance-creation' stamp: 'lr 4/19/2008 11:21'!
on: aBlock message: aString negated: aNegatedBlock message: aNegatedString
	^ self new initializeOn: aBlock message: aString negated: aNegatedBlock message: aNegatedString! !

!PPPredicateParser class methodsFor: 'factory-chars' stamp: 'lr 4/19/2008 11:36'!
space
	^ self 
		on: [ :each | each isSeparator ] message: 'separator expected'
		negated: [ :each | each isSeparator not ] message: 'no separator expected'! !

!PPPredicateParser class methodsFor: 'factory-chars' stamp: 'lr 4/19/2008 11:29'!
uppercase
	^ self lowercase not! !

!PPPredicateParser class methodsFor: 'factory-chars' stamp: 'lr 4/19/2008 11:37'!
word
	^ self 
		on: [ :each | each isAlphaNumeric ] message: 'letter or digit expected'
		negated: [ :each | each isAlphaNumeric not ] message: 'no letter or digit expected'! !

!PPPredicateParser methodsFor: 'parsing' stamp: 'lr 4/21/2008 16:26'!
basicParse: aStream
	^ (aStream atEnd not and: [ predicate value: aStream peek ])
		ifFalse: [ PPFailure reason: predicateMessage position: aStream position ]
		ifTrue: [ aStream next ]! !

!PPPredicateParser methodsFor: 'initialization' stamp: 'lr 4/19/2008 11:23'!
initializeOn: aBlock message: aString negated: aNegatedBlock message: aNegatedString 
	predicate := aBlock.
	predicateMessage := aString.
	negated := aNegatedBlock.
	negatedMessage := aNegatedString! !

!PPPredicateParser methodsFor: 'operators' stamp: 'lr 4/20/2008 15:53'!
not
	"Negate the receiving predicate parser."
	
	^ PPPredicateParser on: negated message: negatedMessage negated: predicate message: predicateMessage! !

PPParser subclass: #PPUnresolvedParser
	instanceVariableNames: 'symbol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Builder'!

!PPUnresolvedParser class methodsFor: 'instance-creation' stamp: 'lr 4/21/2008 07:06'!
on: aSymbol
	^ self new initializeOn: aSymbol! !

!PPUnresolvedParser methodsFor: 'initialization' stamp: 'lr 4/21/2008 07:07'!
initializeOn: aSymbol
	symbol := aSymbol! !

!PPUnresolvedParser methodsFor: 'testing' stamp: 'lr 4/20/2008 16:29'!
isUnresolved
	^ true! !

!PPUnresolvedParser methodsFor: 'parsing' stamp: 'lr 4/22/2008 17:02'!
parse: aStream
	self error: self printString , ' need to be resolved before execution.'! !

!PPUnresolvedParser methodsFor: 'accessing' stamp: 'lr 4/21/2008 07:07'!
symbol
	^ symbol! !

!Interval methodsFor: '*petitparser' stamp: 'lr 4/19/2008 13:06'!
asParser
	^ PPPredicateParser between: start and: stop! !

TestCase subclass: #PPCompositeParserTest
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Tests'!

!PPCompositeParserTest methodsFor: 'utilities' stamp: 'lr 4/22/2008 22:56'!
assert: aCollection fails: aString at: anInteger
	| stream result |
	stream := PPStream on: aCollection.
	result := parser parse: stream.
	self assert: (result isFailure).
	self assert: (result position = anInteger).
	self assert: (result reason includesSubstring: aString)! !

!PPCompositeParserTest methodsFor: 'utilities' stamp: 'lr 4/22/2008 22:57'!
assert: aCollection is: anObject
	| stream result |
	stream := PPStream on: aCollection.
	result := parser parse: stream.
	self assert: result = anObject! !

!PPCompositeParserTest methodsFor: 'accessing' stamp: 'lr 4/21/2008 09:24'!
parseClass
	self subclassResponsibility! !

!PPCompositeParserTest methodsFor: 'running' stamp: 'lr 4/21/2008 09:24'!
setUp
	super setUp.
	parser := self parseClass new! !

PPCompositeParserTest subclass: #PPExpressionParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Tests'!

!PPExpressionParserTest methodsFor: 'accessing' stamp: 'lr 4/21/2008 09:25'!
parseClass
	^ PPExpressionParser! !

!PPExpressionParserTest methodsFor: 'testing' stamp: 'lr 4/21/2008 10:18'!
testAdd
	self assert: '1 + 2' is: 3.
	self assert: '2 + 1' is: 3.
	self assert: '1 + 2.3' is: 3.3.
	self assert: '2.3 + 1' is: 3.3.
	self assert: '1 + -2' is: -1.
	self assert: '-2 + 1' is: -1! !

!PPExpressionParserTest methodsFor: 'testing' stamp: 'lr 4/21/2008 10:23'!
testAddMany
	self assert: '1' is: 1.
	self assert: '1 + 2' is: 3.
	self assert: '1 + 2 + 3' is: 6.
	self assert: '1 + 2 + 3 + 4' is: 10.
	self assert: '1 + 2 + 3 + 4 + 5' is: 15! !

!PPExpressionParserTest methodsFor: 'testing-expression' stamp: 'lr 4/21/2008 10:03'!
testBrackets
	self assert: '(1)' is: 1.
	self assert: '(1 + 2)' is: 3.
	
	self assert: '((1))' is: 1.
	self assert: '((1 + 2))' is: 3.

	self assert: '2 * (3 + 4)' is: 14.
	self assert: '(2 + 3) * 4' is: 20.
	self assert: '6 / (2 + 4)' is: 1.
	self assert: '(2 + 6) / 2' is: 4! !

!PPExpressionParserTest methodsFor: 'testing' stamp: 'lr 4/21/2008 09:32'!
testDiv
	self assert: '12 / 3' is: 4.
	self assert: '-16 / -4' is: 4! !

!PPExpressionParserTest methodsFor: 'testing' stamp: 'lr 4/21/2008 09:31'!
testMul
	self assert: '2 * 3' is: 6.
	self assert: '2 * -4' is: -8! !

!PPExpressionParserTest methodsFor: 'testing' stamp: 'lr 4/21/2008 10:16'!
testMulMany
	self assert: '1 * 2' is: 2.
	self assert: '1 * 2 * 3' is: 6.
	self assert: '1 * 2 * 3 * 4' is: 24.
	self assert: '1 * 2 * 3 * 4 * 5' is: 120! !

!PPExpressionParserTest methodsFor: 'testing' stamp: 'lr 4/21/2008 09:32'!
testNum
	self assert: '0' is: 0.
	self assert: '0.0' is: 0.0.
	self assert: '1' is: 1.
	self assert: '1.2' is: 1.2.
	self assert: '34' is: 34.
	self assert: '56.78' is: 56.78.
	self assert: '-9' is: -9.
	self assert: '-9.9' is: -9.9! !

!PPExpressionParserTest methodsFor: 'testing-expression' stamp: 'lr 4/21/2008 10:00'!
testPriority
	self assert: '2 * 3 + 4' is: 10.
	self assert: '2 + 3 * 4' is: 14.
	self assert: '6 / 3 + 4' is: 6.
	self assert: '2 + 6 / 2' is: 5! !

!PPExpressionParserTest methodsFor: 'testing' stamp: 'lr 4/21/2008 10:00'!
testSub
	self assert: '1 - 2' is: -1.
	self assert: '1.3 - 2' is: -0.7.
	self assert: '1 - -2' is: 3! !

PPCompositeParserTest subclass: #PPLambdaParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Tests'!

!PPLambdaParserTest methodsFor: 'accessing' stamp: 'lr 4/21/2008 10:41'!
parseClass
	^ PPLambdaParser! !

!PPLambdaParserTest methodsFor: 'testing' stamp: 'lr 4/24/2008 10:28'!
testAbstraction
	self assert: '\x.y' is: #($\ 'x' $. 'y').
	self assert: '\x.\y.z' is: #($\ 'x' $. ($\ 'y' $. 'z'))! !

!PPLambdaParserTest methodsFor: 'testing' stamp: 'lr 4/24/2008 10:29'!
testApplication
	self assert: 'x x' is: #('x' 'x').
	self assert: 'x y' is: #('x' 'y').
	self assert: 'x y z' is: #(('x' 'y') 'z')! !

!PPLambdaParserTest methodsFor: 'testing' stamp: 'lr 4/24/2008 10:28'!
testVariable
	self assert: 'x' is: 'x'.
	self assert: 'xy' is: 'xy'! !

PPCompositeParserTest subclass: #PPSchemeParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Tests'!

!PPSchemeParserTest methodsFor: 'accessing' stamp: 'lr 4/24/2008 10:30'!
parseClass
	^ PPSchemeParser! !

TestCase subclass: #PPLibraryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitParser-Tests'!

!PPLibraryTest methodsFor: 'utilities' stamp: 'lr 4/21/2008 16:21'!
assert: aParser fail: aCollection
	| stream result |
	stream := PPStream on: aCollection.
	result := aParser parse: stream.
	self assert: result isFailure.
	self assert: stream position = 0! !

!PPLibraryTest methodsFor: 'utilities' stamp: 'lr 4/21/2008 16:21'!
assert: aParser parse: aCollection to: anObject
	| stream result |
	stream := PPStream on: aCollection.
	result := aParser parse: stream.
	self assert: result = anObject.
	self assert: stream atEnd! !

!PPLibraryTest methodsFor: 'examples' stamp: 'lr 4/20/2008 19:20'!
comment
	^ ($" asParser , $" asParser not star , $" asParser) token! !

!PPLibraryTest methodsFor: 'examples' stamp: 'lr 4/20/2008 19:21'!
identifier
	^ (#letter asParser , #word asParser star) token! !

!PPLibraryTest methodsFor: 'examples' stamp: 'lr 4/22/2008 17:50'!
integer
	^ #digit asParser plus! !

!PPLibraryTest methodsFor: 'examples' stamp: 'lr 4/20/2008 19:21'!
number
	^ ($- asParser optional , #digit asParser plus , ($. asParser , #digit asParser plus) optional) token! !

!PPLibraryTest methodsFor: 'testing' stamp: 'lr 4/20/2008 19:28'!
testAction
	| parser |
	parser := #any asParser ==> [ :a | a ].
	self assert: parser parse: 'a' to: $a.
	self assert: parser parse: 'b' to: $b.
	self assert: parser fail: ''! !

!PPLibraryTest methodsFor: 'testing-extension' stamp: 'lr 4/19/2008 14:06'!
testCharacter
	| parser |
	parser := $a asParser.	
	self assert: parser parse: 'a' to: $a.
	self assert: parser fail: ''.
	self assert: parser fail: 'b'.
	self assert: parser fail: 'A'! !

!PPLibraryTest methodsFor: 'testing' stamp: 'lr 4/20/2008 19:26'!
testChoice
	| parser |
	parser := $a asParser | $b asParser.
	self assert: parser parse: 'a' to: $a.
	self assert: parser parse: 'b' to: $b.
	self assert: parser fail: ''.
	self assert: parser fail: 'c'.
	self assert: parser fail: 'ca'! !

!PPLibraryTest methodsFor: 'testing' stamp: 'lr 4/20/2008 18:58'!
testEndOfInput
	| parser |
	parser := PPEndOfInputParser new.
	self assert: parser parse: '' to: nil.
	self assert: parser fail: 'a'! !

!PPLibraryTest methodsFor: 'testing' stamp: 'lr 4/19/2008 14:05'!
testEpsilon
	| parser |
	parser := PPEpsilonParser new.
	self assert: parser parse: '' to: nil! !

!PPLibraryTest methodsFor: 'testing' stamp: 'lr 4/20/2008 18:58'!
testFailing
	| parser |
	parser := PPFailingParser new.
	self assert: parser fail: ''.
	self assert: parser fail: 'a'.
	self assert: parser fail: 'aa'! !

!PPLibraryTest methodsFor: 'testing-extension' stamp: 'lr 4/19/2008 13:09'!
testInterval
	| parser |
	parser := (1 to: 3) asParser.

	self assert: parser parse: #(1) to: 1.
	self assert: parser parse: #(2) to: 2.
	self assert: parser parse: #(3) to: 3.

	self assert: parser fail: #(0).
	self assert: parser fail: #(4)! !

!PPLibraryTest methodsFor: 'testing-abstract' stamp: 'lr 4/21/2008 16:07'!
testLeftRecursion
	| parser |
	parser := PPChoiceParser new.
	parser | (parser , #digit asParser ==> [ :node | 10 * node first + node second digitValue]).
	parser | (#digit asParser ==> [ :node | node digitValue ]).
	
	! !

!PPLibraryTest methodsFor: 'testing' stamp: 'lr 4/19/2008 14:05'!
testOptional
	| parser |
	parser := $a asParser optional.
	self assert: parser parse: '' to: nil.
	self assert: parser parse: 'a' to: $a! !

!PPLibraryTest methodsFor: 'testing-extension' stamp: 'lr 4/19/2008 13:27'!
testOrdered
	| parser |
	parser := #(1 3) asParser.

	self assert: parser parse: #(1 3) to: #(1 3).

	self assert: parser fail: #().
	self assert: parser fail: #(1).
	self assert: parser fail: #(1 2).
	self assert: parser fail: #(1 1 3)! !

!PPLibraryTest methodsFor: 'testing-abstract' stamp: 'lr 4/26/2008 15:50'!
testParseAaaBbb
	"S0 ::= S1 | epsilon
	 S1 ::= a S0 b "
	
	| s0 s1 parser |
	s0 := (s1 := PPSequenceParser new) | PPEpsilonParser new.
	s1 , $a asParser , s0 , $b asParser.
	parser := s0 end token.

	self assert: parser parse: 'ab' to: 'ab'.
	self assert: parser parse: 'aabb' to: 'aabb'.
	self assert: parser parse: 'aaabbb' to: 'aaabbb'.
	self assert: parser parse: 'aaaabbbb' to: 'aaaabbbb'.
	

	self assert: parser fail: 'a'.
	self assert: parser fail: 'b'.
	self assert: parser fail: 'aab'.
	self assert: parser fail: 'abb'.
	self assert: parser fail: 'aaabb'.
	self assert: parser fail: 'aabbb'! !

!PPLibraryTest methodsFor: 'testing-abstract' stamp: 'lr 4/26/2008 15:51'!
testParseAandB
	"S ::= (A B)+"
	
	| parser |
	parser := ($a asParser , $b asParser) plus token end.

	self assert: parser parse: 'ab' to: 'ab'.
	self assert: parser parse: 'abab' to: 'abab'.
	self assert: parser parse: 'ababab' to: 'ababab'.
	self assert: parser parse: 'abababab' to: 'abababab'.
	
	self assert: parser fail: ''.
	self assert: parser fail: 'a'.
	self assert: parser fail: 'aba'.
	self assert: parser fail: 'ababa'.
	self assert: parser fail: 'ababbbb'! !

!PPLibraryTest methodsFor: 'testing-abstract' stamp: 'lr 4/26/2008 15:51'!
testParseAorB
	"S ::= (A | B)+"

	| parser |
	parser := ($a asParser | $b asParser) plus token end.

	self assert: parser parse: 'a' to: 'a'.
	self assert: parser parse: 'b' to: 'b'.
	self assert: parser parse: 'ab' to: 'ab'.
	self assert: parser parse: 'ba' to: 'ba'.
	self assert: parser parse: 'aaa' to: 'aaa'.
	self assert: parser parse: 'aab' to: 'aab'.
	self assert: parser parse: 'aba' to: 'aba'.
	self assert: parser parse: 'baa' to: 'baa'.
	self assert: parser parse: 'abb' to: 'abb'.
	self assert: parser parse: 'bab' to: 'bab'.
	self assert: parser parse: 'bba' to: 'bba'.
	self assert: parser parse: 'bbb' to: 'bbb'.
	
	self assert: parser fail: ''.
	self assert: parser fail: 'c'.
	self assert: parser fail: 'abc'! !

!PPLibraryTest methodsFor: 'testing-examples' stamp: 'lr 4/20/2008 18:59'!
testParseComment
	self assert: self comment parse: '""' to: '""'.
	self assert: self comment parse: '"a"' to: '"a"'.
	self assert: self comment parse: '"ab"' to: '"ab"'.
	self assert: self comment parse: '"abc"' to: '"abc"'.

	self assert: self comment fail: '"'.
	self assert: self comment fail: '"a'.
	self assert: self comment fail: '"aa'.
	self assert: self comment fail: 'a"'.
	self assert: self comment fail: 'aa"'! !

!PPLibraryTest methodsFor: 'testing-examples' stamp: 'lr 4/19/2008 13:52'!
testParseIdentifier
	self assert: self identifier parse: 'a' to: 'a'.
	self assert: self identifier parse: 'a1' to: 'a1'.
	self assert: self identifier parse: 'a12' to: 'a12'.
	self assert: self identifier parse: 'ab' to: 'ab'.
	self assert: self identifier parse: 'a1b' to: 'a1b'.
	
	self assert: self identifier fail: ''.
	self assert: self identifier fail: ' '.
	self assert: self identifier fail: '1'.
	self assert: self identifier fail: '1a'! !

!PPLibraryTest methodsFor: 'testing-abstract' stamp: 'lr 4/26/2008 15:51'!
testParseListOfNumbersLeftRecursive
	"S ::= S , number | number"
	
	| number parser |
	number := self number ==> [ :node | Number readFrom: node copy ].
	parser := PPChoiceParser new.
	parser | (parser , $, asParser token , number ==> [ :node | node first copyWith: node last ]).
	parser | (number ==> [ :node | Array with: node ]).
	parser := parser end.

	self assert: parser parse: '1' to: (1 to: 1) asArray.
	self assert: parser parse: '1, 2' to: (1 to: 2) asArray.
	self assert: parser parse: '1, 2, 3' to: (1 to: 3) asArray.
	self assert: parser parse: '1, 2, 3, 4' to: (1 to: 4) asArray.
	self assert: parser parse: '1, 2, 3, 4, 5' to: (1 to: 5) asArray! !

!PPLibraryTest methodsFor: 'testing-examples' stamp: 'lr 4/19/2008 14:06'!
testParseNumber
	self assert: self number parse: '1' to: '1'.
	self assert: self number parse: '12' to: '12'.
	self assert: self number parse: '12.3' to: '12.3'.
	self assert: self number parse: '12.34' to: '12.34'.
	
	self assert: self number parse: '-1' to: '-1'.
	self assert: self number parse: '-12' to: '-12'.
	self assert: self number parse: '-12.3' to: '-12.3'.
	self assert: self number parse: '-12.34' to: '-12.34'.
	
	self assert: self number fail: ''.
	self assert: self number fail: '-'.
	self assert: self number fail: '.'.
	self assert: self number fail: '.1'! !

!PPLibraryTest methodsFor: 'testing-examples' stamp: 'lr 4/20/2008 18:59'!
testParseReturn
	| number spaces return |
	number := #digit asParser plus token.
	spaces := #space asParser star.
	return := (spaces , $^ asParser , spaces , number , spaces)
		==> [ :nodes | { #return. nodes at: 4 } ].

	self assert: return parse: '^1' to: #(return '1').
	self assert: return parse: ' ^12' to: #(return '12').
	self assert: return parse: '^123 ' to: #(return '123').
	self assert: return parse: '^ 1234' to: #(return '1234')! !

!PPLibraryTest methodsFor: 'testing' stamp: 'lr 4/19/2008 13:59'!
testPlus
	| parser |
	parser := $a asParser plus.
	self assert: parser parse: 'a' to: #($a).
	self assert: parser parse: 'aa' to: #($a $a).
	self assert: parser parse: 'aaa' to: #($a $a $a).
	self assert: parser fail: ''.
	self assert: parser fail: 'b'.
	self assert: parser fail: 'ba'! !

!PPLibraryTest methodsFor: 'testing-predicate' stamp: 'lr 4/20/2008 19:00'!
testPredicateAny
	| parser |
	parser := #any asParser.

	self assert: parser parse: ' ' to: $ .
	self assert: parser parse: '1' to: $1.
	self assert: parser parse: 'a' to: $a.
	
	self assert: parser fail: ''.
	
	self assert: parser not fail: ''.
	self assert: parser not fail: '1'.
	self assert: parser not fail: 'a'! !

!PPLibraryTest methodsFor: 'testing-predicate' stamp: 'lr 4/20/2008 19:10'!
testPredicateCharacter
	| parser |
	parser := $* asParser.
	
	self assert: parser parse: '*' to: $*.
	
	self assert: parser fail: ''.
	self assert: parser fail: '1'.
	self assert: parser fail: 'a'.
	
	self assert: parser not parse: '1' to: $1.
	self assert: parser not parse: 'a' to: $a.
	
	self assert: parser not fail: '*'.
	self assert: parser not fail: ''
	! !

!PPLibraryTest methodsFor: 'testing-predicate' stamp: 'lr 4/20/2008 19:09'!
testPredicateControl
	| parser |
	parser := #control asParser.

	self assert: parser parse: String cr to: Character cr.
	self assert: parser parse: String tab to: Character tab.
	
	self assert: parser fail: ''.
	self assert: parser fail: '1'.
	self assert: parser fail: 'a'.
	
	self assert: parser not parse: '1' to: $1.
	self assert: parser not parse: 'a' to: $a.
	
	self assert: parser not fail: ''.
	self assert: parser not fail: String cr.
	self assert: parser not fail: String tab! !

!PPLibraryTest methodsFor: 'testing-predicate' stamp: 'lr 4/20/2008 19:09'!
testPredicateDigit
	| parser |
	parser := #digit asParser.
	
	self assert: parser parse: '0' to: $0.
	self assert: parser parse: '9' to: $9.

	self assert: parser fail: ''.
	self assert: parser fail: 'a'.
	
	self assert: parser not parse: ' ' to: $ .
	self assert: parser not parse: 'a' to: $a.

	self assert: parser not fail: ''.
	self assert: parser not fail: '0'! !

!PPLibraryTest methodsFor: 'testing-predicate' stamp: 'lr 4/20/2008 19:09'!
testPredicateLetter
	| parser |
	parser := #letter asParser.

	self assert: parser parse: 'a' to: $a.
	self assert: parser parse: 'Z' to: $Z.

	self assert: parser fail: ''.
	self assert: parser fail: '0'.
	
	self assert: parser not parse: '1' to: $1.
	self assert: parser not parse: ' ' to: $ .

	self assert: parser not fail: ''.
	self assert: parser not fail: 'a'! !

!PPLibraryTest methodsFor: 'testing-predicate' stamp: 'lr 4/20/2008 19:09'!
testPredicateLowercase
	| parser |
	parser := #lowercase asParser.
	
	self assert: parser parse: 'a' to: $a.
	self assert: parser parse: 'z' to: $z.

	self assert: parser fail: ''.
	self assert: parser fail: 'A'.
	self assert: parser fail: '0'! !

!PPLibraryTest methodsFor: 'testing-predicate' stamp: 'lr 4/20/2008 19:09'!
testPredicateSpace
	| parser |
	parser := #space asParser.
	
	self assert: parser parse: String tab to: Character tab.
	self assert: parser parse: ' ' to: Character space.

	self assert: parser fail: ''.
	self assert: parser fail: 'a'.
	
	self assert: parser not parse: 'a' to: $a.
	self assert: parser not parse: '/' to: $/.

	self assert: parser not fail: ''.
	self assert: parser not fail: ' '.! !

!PPLibraryTest methodsFor: 'testing-predicate' stamp: 'lr 4/20/2008 19:10'!
testPredicateUppercase
	| parser |
	parser := #uppercase asParser.
	
	self assert: parser parse: 'A' to: $A.
	self assert: parser parse: 'Z' to: $Z.

	self assert: parser fail: ''.
	self assert: parser fail: 'a'.
	self assert: parser fail: '0'! !

!PPLibraryTest methodsFor: 'testing-predicate' stamp: 'lr 4/20/2008 19:10'!
testPredicateWord
	| parser |
	parser := #word asParser.

	self assert: parser parse: 'a' to: $a.
	self assert: parser parse: 'A' to: $A.
	self assert: parser parse: '0' to: $0.

	self assert: parser fail: ''.
	self assert: parser fail: '-'.
	
	self assert: parser not parse: ' ' to: $ .
	self assert: parser not parse: '-' to: $-.

	self assert: parser not fail: ''.
	self assert: parser not fail: 'a'! !

!PPLibraryTest methodsFor: 'testing' stamp: 'lr 4/20/2008 19:26'!
testSequence
	| parser |
	parser := $a asParser , $b asParser.
	self assert: parser parse: 'ab' to: #($a $b).
	self assert: parser fail: ''.
	self assert: parser fail: 'a'.
	self assert: parser fail: 'aa'.
	self assert: parser fail: 'ba'.
	self assert: parser fail: 'bab'! !

!PPLibraryTest methodsFor: 'testing' stamp: 'lr 4/19/2008 14:04'!
testStar
	| parser |
	parser := $a asParser star.
	self assert: parser parse: '' to: #().
	self assert: parser parse: 'a' to: #($a).
	self assert: parser parse: 'aa' to: #($a $a).
	self assert: parser parse: 'aaa' to: #($a $a $a)! !

!PPLibraryTest methodsFor: 'testing-extension' stamp: 'lr 4/20/2008 19:18'!
testString
	| parser |
	parser := 'ab' asParser.

	self assert: parser parse: 'ab' to: #($a $b).

	self assert: parser fail: 'a'.
	self assert: parser fail: 'ac'! !

!PPLibraryTest methodsFor: 'testing' stamp: 'lr 4/21/2008 08:01'!
testToken
	| parser |
	parser := $a asParser token.

	self assert: parser parse: 'a' to: 'a'.
	self assert: parser parse: 'a ' to: 'a'.
	self assert: parser parse: 'a  ' to: 'a'.

	self assert: parser fail: ''.
	self assert: parser fail: 'b'! !

!PPLibraryTest methodsFor: 'testing-extension' stamp: 'lr 4/19/2008 13:27'!
testUnordered
	| parser |
	parser := #(1 3) asSet asParser.

	self assert: parser parse: #(1) to: 1.
	self assert: parser parse: #(3) to: 3.

	self assert: parser fail: #().
	self assert: parser fail: #(2).
	self assert: parser fail: #(2 1).
	self assert: parser fail: #(2 3)! !
